package logic

import (
	"encoding/json"
	"errors"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"

	"context"

	"fusenapi/server/ldap-admin/internal/svc"
	"fusenapi/server/ldap-admin/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
	"gorm.io/gorm"
)

type SetLdapGroupAuthLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewSetLdapGroupAuthLogic(ctx context.Context, svcCtx *svc.ServiceContext) *SetLdapGroupAuthLogic {
	return &SetLdapGroupAuthLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *SetLdapGroupAuthLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

func (l *SetLdapGroupAuthLogic) SetLdapGroupAuth(req *types.SetLdapGroupAuthReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	// 返回值必须调用Set重新返回, resp可以空指针调用 resp.SetStatus(basic.CodeOK, data)
	// userinfo 传入值时, 一定不为null
	resOne, err := l.svcCtx.AllModels.LdapGroup.FindOneById(l.ctx, req.GroupId)
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			basic.CodeServiceErr.Message = "记录不存在"
		} else {
			basic.CodeServiceErr.Message = "系统出错"
		}
		return resp.SetStatus(basic.CodeServiceErr)
	}

	var metadata []types.GroupAuth
	if resOne.Metadata != nil {
		err := json.Unmarshal(*resOne.Metadata, &metadata)
		if err != nil {
			basic.CodeServiceErr.Message = "系统出错"
			return resp.SetStatus(basic.CodeServiceErr)
		}
	}
	var groupAuth = types.GroupAuth{
		Id:       req.GroupAuth.Id,
		Type:     req.GroupAuth.Type,
		Name:     req.GroupAuth.Name,
		Metadata: req.GroupAuth.Metadata,
	}
	var status = 0
	if len(metadata) > 0 {
		for k, v := range metadata {
			if v.Type == groupAuth.Type && v.Id == groupAuth.Id {
				status = 1
				metadata[k] = groupAuth
			}
		}
	}
	if status == 0 {
		metadata = append(metadata, groupAuth)
	}
	// 更新metadata
	metadataByte, _ := json.Marshal(metadata)
	err = l.svcCtx.AllModels.LdapGroup.UpdateOne(l.ctx, resOne, map[string]interface{}{
		"metadata": string(metadataByte),
	})
	if err != nil {
		basic.CodeServiceErr.Message = "系统出错"
		return resp.SetStatus(basic.CodeServiceErr)
	}
	return resp.SetStatus(basic.CodeOK)
}

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *SetLdapGroupAuthLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }
